/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient.datacomponents;

//---IMPORTS---
import javax.swing.JLabel;
import java.awt.GridLayout;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;
import uk.org.iscream.cms.util.XMLPacket;

/**
 * This is DataComponent specifically for 
 * displaying CPU percentages.
 *
 * It uses the JProgressBar to display the value.
 *
 * @author  $Author: tdb $
 * @version $Id: CPUDataComponent.java,v 1.14 2003/02/05 19:35:04 tdb Exp $
 */
public class CPUDataComponent extends VisibleDataComponent {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.14 $";
    
//---STATIC METHODS---
    
//---CONSTRUCTORS---

    /**
     * Creates the component with a friendly name to be 
     * used as label
     *
     * @param name the friendly name
     * @param attribute the data attribute we look after
     */
    public CPUDataComponent(String name, String attribute) {
        _name = name;
        _attribute = attribute;
        _label = new JLabel(_name + ": ");
        _label.setHorizontalAlignment(JLabel.RIGHT);
        setLayout(new GridLayout(1, 2));
	    _item.setString("-uninitialised-");
        add(_label);
        add(_item);
        setVisible(false);
    }

//---PUBLIC METHODS---

    /**
     * This run method updates any Swing components
     * The setValue() method adds this component
     * to the Swing Event Dispatching Queue to
     * run this method.
     */
    public void run() {
        if(!isVisible()) {
            setVisible(true);
        }
        _item.setString(_cache + "%");
        _item.setValue(new Double(_cache).intValue());
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return _name + "(" + _attribute + ")";
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

    /**
     * This takes the packet to obtain the value from, it then performs all
     * approriate conversions and adds this class to the Swing Event
     * Dispatching queue.
     *
     * @param packet the XMLPacket to get the data from
     * @throws DataFormatException if there was a problem converting the data for display
     */
    public void setValue(XMLPacket packet) throws DataFormatException {
        String value = packet.getParam(_attribute);
        try {
            if(!_cache.equals(value)) {
                _cache = value;
                SwingUtilities.invokeLater(this);
            }
        } catch (Exception e) {
            throw new DataFormatException(value + " is an invalid data type for " + toString());
        }
    }
    
    /**
     * Returns the string showing the packet
     * attribute that the component is looking after
     *
     * @return the packet reference
     */
    public String getPacketAttribute() {
        return _attribute;
    }

//---ATTRIBUTES---

    /**
     * The friendly name of this component
     */
    private String _name;
    
    /**
     * The attribute that this component is concerned with
     */
    private String _attribute;
    
    /**
     * Remembers what the last value was, so we
     * only update if we have to.
     */    
    protected String _cache = "";
    
    /**
     * The minimum value for the percentage
     */
    private final int _min = 0;
    
    /**
     * The maximum value for the percentage
     */
    private final int _max = 100;
    
    /**
     * The friendly label for this component
     */
    private JLabel _label;
    
    /**
     * The progress bar that we will display CPU
     * percentage data in
     */
    private JProgressBar _item = new JProgressBar(JProgressBar.HORIZONTAL, _min, _max);
    {
        _item.setStringPainted(true);
    }

//---STATIC ATTRIBUTES---

}
