/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient.datacomponents;

//---IMPORTS---
import javax.swing.JLabel;
import java.awt.GridLayout;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;
import uk.org.iscream.cms.util.XMLPacket;

/**
 * This is DataComponent specifically for 
 * displaying Disk information.  It displays
 * a value/total type display using a JProgessBar.
 *
 * It does this by extending the StorageDataComponent.
 *
 * @author  $Author: tdb $
 * @version $Id: DiskDataComponent.java,v 1.5 2003/02/05 19:35:04 tdb Exp $
 */
public class DiskDataComponent extends StorageDataComponent {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.5 $";
    
//---STATIC METHODS---
    
//---CONSTRUCTORS---

    /**
     * Creates the component with a friendly name to be 
     * used as label
     *
     * @param name the friendly name
     * @param attribute the data attribute we look after
     * @param maxAttribute the data attribute to obtain the maximum value from
     * @param mountAttribute the data attribute to obtain the disk mount point
     * @param devAttribute the data attribute to obtain the disk device
     * @param unit the string representation of the units eg, "Mb" or "Kb"
     * @param divider the amount that the given value should be multiplied by to reach the given units
     * @param mountPoint the component that will supply the mount point for this disk
     * @param device the component that will supply the device for this disk
     */
    public DiskDataComponent(String name, String attribute, String maxAttribute, String mountAttribute, String deviceAttribute, String unit, int divider) {
        super(name, attribute, maxAttribute, unit, divider);
        _mountAttribute = mountAttribute;
        _deviceAttribute = deviceAttribute;
    }

//---PUBLIC METHODS---
    
    /**
     * This run method updates any Swing components
     * The setValue() method adds this component
     * to the Swing Event Dispatching Queue to
     * run this method.
     */
    public void run() {
        super.run();
        setLabel();
    }
    
//---PRIVATE METHODS---

    private void setLabel() {
        _label.setText(_name + " on " + _mountCache + " (" + _deviceCache + ") : ");
    }

//---ACCESSOR/MUTATOR METHODS---

   /**
     * This takes the packet to obtain the value from, it then performs all
     * approriate conversions and adds this class to the Swing Event
     * Dispatching queue.
     *
     * @param packet the XMLPacket to get the data from
     * @throws DataFormatException if there was a problem converting the data for display
     */
    public void setValue(XMLPacket packet) throws DataFormatException {
        super.setValue(packet);
        String mountvalue = packet.getParam(_mountAttribute);
        String devicevalue = packet.getParam(_deviceAttribute);

        if (!_mountCache.equals(mountvalue) || !_deviceCache.equals(devicevalue)) {
            _mountCache = mountvalue;
            _deviceCache = devicevalue;
            SwingUtilities.invokeLater(this);
        }
    }        

//---ATTRIBUTES---

    /**
     * Holds the last value of the mountPoint
     */
    private String _mountCache = "";
        
    /**
     * Hold the last value for the device
     */
    private String _deviceCache = "";
    
    /**
     * The device attribute that this component is concerned with
     */
    private String _deviceAttribute;
    
    /**
     * The mount attribute that this component is concerned with
     */
    private String _mountAttribute;

//---STATIC ATTRIBUTES---

}
