/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient.datacomponents;

//---IMPORTS---
import javax.swing.JLabel;
import java.awt.GridLayout;
import java.util.StringTokenizer;
import java.util.Arrays;
import javax.swing.JComboBox;
import javax.swing.SwingUtilities;
import uk.org.iscream.cms.util.XMLPacket;

/**
 * This component displays the users currently logged
 * onto a system.  Currently it does this simply by 
 * putting them all in a JCombobox.
 *
 * @author  $Author: tdb $
 * @version $Id: UsersDataComponent.java,v 1.20 2003/02/05 19:35:04 tdb Exp $
 */
public class UsersDataComponent extends VisibleDataComponent {

//---FINAL ATTRIBUTES---

//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Creates the component with a friendly name to be 
     * used as label, but as we're a very basic
     * extension of StringDataComponent, we just construct
     * that.
     *
     * @param name the friendly name
     * @param attribute the data attribute we look after
     */
    public UsersDataComponent(String name, String attribute) {
        _name = name;
        _attribute = attribute;
        _label = new JLabel(name + ": ");
        _label.setHorizontalAlignment(JLabel.RIGHT);
        setLayout(new GridLayout(1, 2));
        _item.addItem("-uninitialised-");
        add(_label);
        add(_item);
        setVisible(false);
    }

//---PUBLIC METHODS---
    
    /**
     * This run method updates any Swing components
     * The setValue() method adds this component
     * to the Swing Event Dispatching Queue to
     * run this method.
     */
    public void run() {
        if(!isVisible()) {
            setVisible(true);
        }
	    _item.removeAllItems();
	    if (_cache.equals(" ")) {
	        _item.addItem("no users logged on");
        } else {
            StringTokenizer st = new StringTokenizer(_cache, ";");
            while(st.hasMoreTokens()) {
                _item.addItem(st.nextToken());
            }
        }
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return _name + "(" + _attribute + ")";
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---
     
   /**
     * This takes the packet to obtain the value from, it then performs all
     * approriate conversions and adds this class to the Swing Event
     * Dispatching queue.
     *
     * @param packet the XMLPacket to get the data from
     * @throws DataFormatException if there was a problem converting the data for display
     */
    public void setValue(XMLPacket packet) throws DataFormatException {
        String value = packet.getParam(_attribute);
        // tokenize the input
        StringTokenizer st = new StringTokenizer(value);
        // create an array to store the tokens in
        int tokenCount = st.countTokens();
        String[] tokens = new String[tokenCount];
        int i=0;
        while(st.hasMoreTokens()) {
            tokens[i] = st.nextToken();
            i++;
        }
        // sort the array
        Arrays.sort(tokens);
        // put the array back into a String, sorted
        StringBuffer valueBuffer = new StringBuffer();
        // we only add the token when we've gone past all
        // the duplicates - this allows us to keep count.
        String lastToken = "";
        int count = 0;
        for(i=0; i < tokenCount; i++) {
            // if the token is different, we'll add the
            // last one
            if(!tokens[i].equals(lastToken)) {
                // just a check for the "first case"
                if(!lastToken.equals("")) {
                    valueBuffer.append(lastToken);
                    if(count > 1) {
                        valueBuffer.append(" (");
                        valueBuffer.append(count);
                        valueBuffer.append(")");
                    }
                    valueBuffer.append(";");
                }
                // starting a new token, so reset the
                // the last one and the count
                lastToken = tokens[i];
                count = 1;
            }
            // if not, we'll increment our count of the
            // current one
            else {
                count++;
            }
        }
        // we have to add the last one...
        valueBuffer.append(lastToken);
        if(count > 1) {
            valueBuffer.append(" (");
            valueBuffer.append(count);
            valueBuffer.append(");");
        }
        valueBuffer.append(";");
        String sortedValue = valueBuffer.toString();
        try {
            if (!_cache.equals(sortedValue)) {
                _cache = sortedValue;
                SwingUtilities.invokeLater(this);
            }
        } catch (Exception e) {
            throw new DataFormatException(sortedValue + " is an invalid data type for " + toString());
        }
    }
    
    /**
     * Returns the string showing the packet
     * attribute that the component is looking after
     *
     * @return the packet reference
     */
    public String getPacketAttribute() {
        return _attribute;
    }

//---ATTRIBUTES---

    /**
     * The friendly name of this component
     */
    private String _name;
    
    /**
     * The attribute that this component is concerned with
     */
    private String _attribute;
    
    /**
     * Remebers what the last value was, so we
     * only update if we have to.
     */ 
    String _cache = "";

    /**
     * The friendly label for this component
     */
    protected JLabel _label;
    
    /**
     * Just a JComboBox to display the users in
     */
    protected JComboBox _item = new JComboBox();

//---STATIC ATTRIBUTES---

}
